/*
 * Copyright 2017 by A-SIT, Secure Information Technology Center Austria
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */

// Remember the active tab
let activeTab = null;

const keyUsage = [
  { name: "digitalSignature", ui: "Digital Signature" },
  { name: "nonRepudiation",   ui: "Non Repudiation" },
  { name: "keyEncipherment",  ui: "Key Transport" },
  { name: "dataEncipherment", ui: "User Data Encipherment" },
  { name: "keyAgreement",     ui: "Key Agreement" },
  { name: "keyCertSign",      ui: "Signature Verification" },
  { name: "serverAuth",       ui: "TLS Web Server Authentication"},
  { name: "clientAuth",       ui: "TLS Web Client Authentication"},
  { name: "codeSigning",      ui: "Code Signing"},
  { name: "emailProtection",  ui: "E-mail Protection"},
  { name: "timeStamping",     ui: "Timestamping"},
  { name: "OCSPSigning",      ui: "OCSP Signing"}
];

// function to convert string to ArrayBuffer
function str2ab(str) {
  let buf = new ArrayBuffer(str.length);
  let bufView = new Uint8Array(buf);

  for (let i = 0, strLen = str.length; i < strLen; i++)
    bufView[i] = str.charCodeAt(i);

  return buf;
}

function toHexCodes(input_buffer) {
  let result = "";
  let int_buffer = new Uint8Array(input_buffer);

  for (let i = 0; i < int_buffer.length; i++) {
    let str = int_buffer[i].toString(16).toUpperCase();
    result = result + ((str.length === 1) ? '0' : '') + str;
  }

  return result.trim();
}

function formatKeyUsage(extension) {
  let result = '';
  for (let usage of keyUsage) {
    if (extension[usage.name]) {
      result += (result.length > 0 ? ', ' : '') + usage.ui;
    }
  }

  return result;
}

function parseXmlResponse(responseXml) {
  let res = {};
  res['hasSigInfo'] = false;
  res['hasTslInfo'] = false;
  res['hasFaultCode'] = false;
  res['hasErrorCode'] = false;
  res['hasException'] = false;

  let xmlDoc = $.parseXML(responseXml);
  let $xml = $(xmlDoc);

  let sigInfo = $xml.find("ns2\\:SignatureInfo, SignatureInfo");
  if (sigInfo.length) {
    res['hasSigInfo'] = true;
    //let signerName = sigInfo.find("ns2\\:SignerName, SignerName").text();

    let certCheck = $xml.find("ns2\\:CertificateCheck, CertificateCheck");
    //let trustInfo = certCheck.find("ns2\\:Info, Info").text();
    res['isTrusted'] = (certCheck.find("ns2\\:Code, Code").text() == '0');

    // 8814 = Gewöhnliches Zertifikat
    // 8815 = Qualifiziertes Zertifikat
    let annotationCode = $xml.find("ns2\\:Annotation, Annotation").find("ns2\\:Code, Code").text();
    res['quality'] = (annotationCode == '8815' ? 'Qualified certificate' : 'Non-qualified certificate');
    //console.log('Issuer country code: ' + $xml.find("ns2\\:IssuerCountryCode, IssuerCountryCode").text());

    let tslInfo = $xml.find("ns2\\:TSLInformation, TSLInformation");
    // Note: Checking tslInfo.length does not because in case of <ns2:TSLInformation/>
    if (tslInfo.children().length) {
      res['hasTslInfo'] = true;
      res['tsl_issuercountrycode'] = tslInfo.find("ns2\\:TSLIssuerCountryCode, TSLIssuerCountryCode").text();
      res['tsl_servicetypestatus'] = tslInfo.find("ns2\\:ServiceTypeStatus, ServiceTypeStatus").text();
      res['tsl_servicetypeidentifier'] = tslInfo.find("ns2\\:ServiceTypeIdentifier, ServiceTypeIdentifier").text();
      res['tsl_additionalserviceinformation'] = tslInfo.find("ns2\\:AdditionalServiceInformation, AdditionalServiceInformation").text();
    }
  } else if ($xml.find("ns2\\:faultcode, faultcode").length) {
    res['hasFaultCode'] = true;
    res['faultstring'] = $xml.find("ns2\\:faultstring, faultstring").text();

    let errorRes = $xml.find("ns2\\:ErrorResponse, ErrorResponse");
    let exceptionMsg = $xml.find("ns2\\:Exception, Exception");
    if (errorRes.length) {
      res['hasErrorCode'] = true;
      res['errorcode'] = errorRes.find("ns2\\:ErrorCode, ErrorCode").text();
      res['errorinfo'] = errorRes.find("ns2\\:Info, Info").text();
    } else if (exceptionMsg.length) {
      res['hasException'] = true;
      res['exceptionMsg'] = exceptionMsg.text();
    }
  } else {
    // Neither a verification result, nor a faultcode. What now?
    res['hasFaultCode'] = true;
    res['faultstring'] = 'Unknown error!';
  }

  return res;
}

function parseCert(pemCert) {
  let parsedCert = { subject: {}, issuer: {}, validity: {} };
  let asn1 = org.pkijs.fromBER(str2ab(window.atob(pemCert)));
  let cert = new org.pkijs.simpl.CERT({ schema: asn1.result });

  for (let i = 0; i < cert.subject.types_and_values.length; i++) {
    switch (cert.subject.types_and_values[i].type) {
      case '2.5.4.3':
        parsedCert.subject.CN = cert.subject.types_and_values[i].value.value_block.value;
        break;
      case '2.5.4.6':
        parsedCert.subject.C = cert.subject.types_and_values[i].value.value_block.value;
        break;
      case '2.5.4.10':
        parsedCert.subject.OU = cert.subject.types_and_values[i].value.value_block.value;
        break;
      case '2.5.4.11':
        parsedCert.subject.O = cert.subject.types_and_values[i].value.value_block.value;
        break;
      case '1.2.840.113549.1.9.1':
        parsedCert.subject.E = cert.subject.types_and_values[i].value.value_block.value;
        break;
      default:
        break;
    }
  }

  for (let i = 0; i < cert.issuer.types_and_values.length; i++) {
    switch (cert.issuer.types_and_values[i].type) {
      case '2.5.4.3':
        parsedCert.issuer.CN = cert.issuer.types_and_values[i].value.value_block.value;
        break;
      case '2.5.4.6':
        parsedCert.issuer.C = cert.issuer.types_and_values[i].value.value_block.value;
        break;
      case '2.5.4.10':
        parsedCert.issuer.OU = cert.issuer.types_and_values[i].value.value_block.value;
        break;
      case '2.5.4.11':
        parsedCert.issuer.O = cert.issuer.types_and_values[i].value.value_block.value;
        break;
      default:
        break;
    }
  }

  parsedCert.validity.notBefore = cert.notBefore.value;
  parsedCert.validity.notAfter = cert.notAfter.value;

  if (cert.serialNumber.value_block.is_hex_only == true) {
    let serial = toHexCodes(cert.serialNumber.value_block.value_hex);
    parsedCert.serialNumber = serial.match(/.{2}/g).join(':');
  } else {
    parsedCert.serialNumber = cert.serialNumber.value_block.value_dec;
  }

  parsedCert.keyUsage = '';
  for (let i = 0; i < cert.extensions.length; i++) {
    if (cert.extensions[i].extnID == '2.5.29.15') { // Key usage
      let bitArray = cert.extensions[i].parsedValue.value_block.value_hex;
      let bitView = new Uint8Array(bitArray);
      let b2 = bitView[0];

      let e = {};
      e.digitalSignature = (b2 & 0x80) === 0x80;
      e.nonRepudiation = (b2 & 0x40) === 0x40;
      e.keyEncipherment = (b2 & 0x20) === 0x20;
      e.dataEncipherment = (b2 & 0x10) === 0x10;
      e.keyAgreement = (b2 & 0x08) === 0x08;
      e.keyCertSign = (b2 & 0x04) === 0x04;

      let keyUsageFormatted = formatKeyUsage(e);
      parsedCert.keyUsage += (parsedCert.keyUsage.length > 0 ? ', ' : '') + keyUsageFormatted;
    } else if (cert.extensions[i].extnID == '2.5.29.37') { // Extended key usage
      let e = {};
      if (cert.extensions[i].parsedValue.keyPurposes) {
        for (let purpose of cert.extensions[i].parsedValue.keyPurposes) {
          switch (purpose) {
            case '1.3.6.1.5.5.7.3.1':
              e.serverAuth = true;
              break;
            case '1.3.6.1.5.5.7.3.2':
              e.clientAuth = true;
              break;
            case '1.3.6.1.5.5.7.3.3':
              e.codeSigning = true;
              break;
            case '1.3.6.1.5.5.7.3.4':
              e.emailProtection = true;
              break;
            case '1.3.6.1.5.5.7.3.8':
              e.timeStamping = true;
              break;
            case '1.3.6.1.5.5.7.3.9':
              e.OCSPSigning = true;
              break;
            default:
              break;
          }
        }
      }

      let keyUsageFormatted = formatKeyUsage(e);
      parsedCert.keyUsage += (parsedCert.keyUsage.length > 0 ? ', ' : '') + keyUsageFormatted;
    }
  }

  return parsedCert;
}

self.port.on('connData', function onMessage(responseXml, pemCert) {
  // Send close message to hide the panel
  $('.close').click(function () {
    self.postMessage("close");
  });

  let xmlParsed = parseXmlResponse(responseXml);
  let cert = null;
  try {
    cert = parseCert(pemCert);
  } catch (e) {
    // Thrown if invalid cert was sent to server. Likely, responseXml also contains an error description.
    console.log(e.message);
  }

  // Tab: Trust Status
  const icon_no = '<i class="fa fa-times fa-lg"></i>';
  const icon_lock = '<i class="fa fa-lock fa-lg"></i>';
  const icon_unlock = '<i class="fa fa-unlock-alt fa-lg"></i>';
  let status = $('#co-status');
  let errormsg = $('#errormsg');
  errormsg.hide();
  errormsg.empty();
  status.empty();
  if (xmlParsed['isTrusted']) {
    status.append(icon_lock + 'Trusted TLS certificate');
  } else if (xmlParsed['isTrusted'] !== undefined) {
    status.append(icon_unlock + 'Server certificate untrusted');
  } else if (xmlParsed['hasFaultCode']) {
    status.append(icon_no + xmlParsed['faultstring']);
  }

  let chain = $('#ce-chain');
  chain.empty();

  if (xmlParsed['hasSigInfo']) {
    chain.append('<h4 class="headline">' +
      (cert.subject.CN ? cert.subject.CN : '&lt;not provided&gt;') +
      '<span class="certdesc"><i class="fa fa-angle-double-left" aria-hidden="true"></i> Subject</span>' +
      '</h4><table class="table table-condensed table-striped detailinfo"><tbody>' +
      '<tr><td><b>Quality</b></td><td>' + xmlParsed['quality'] + '</td></tr>' +
      (cert.subject.O ? '<tr><td><b>Organization</b></td><td>' + cert.subject.O + '</td></tr>' : '') +
      (cert.subject.OU ? '<tr><td><b>Organization Unit</b></td><td>' + cert.subject.OU + '</td></tr>' : '') +
      (cert.subject.C ? '<tr><td><b>Country</b></td><td>' + cert.subject.C + '</td></tr>' : '') +
      (cert.subject.E ? '<tr><td><b>E-Mail</b></td><td>' + cert.subject.E + '</td></tr>' : '') +
      '<tr><td><b>Validity</b></td><td>' + new Date(cert.validity.notBefore).toLocaleString() + ' &ndash; ' + new Date(cert.validity.notAfter).toLocaleString() + '</td></tr>' +
      //(certNo == 0 ? '<tr><td><b>Extended Validation</b></td><td>' + (mainConn.tlsIsExtendedValidation ? icon_yes : icon_no) + '</td></tr>' : '') +
      '</tbody></table>');

    chain.append('<h4 class="headline certsep">' +
      (cert.issuer.CN ? cert.issuer.CN : '&lt;not provided&gt;') +
      '<span class="certdesc"><i class="fa fa-angle-double-left" aria-hidden="true"></i> Issuer</span>' +
      '</h4><table class="table table-condensed table-striped detailinfo"><tbody>' +
      (cert.issuer.O ? '<tr><td><b>Organization</b></td><td>' + cert.issuer.O + '</td></tr>' : '') +
      (cert.issuer.OU ? '<tr><td><b>Organization Unit</b></td><td>' + cert.issuer.OU + '</td></tr>' : '') +
      (cert.issuer.C ? '<tr><td><b>Country</b></td><td>' + cert.issuer.C + '</td></tr>' : '') +
      '</tbody></table>');
  } else if (xmlParsed['hasErrorCode']) {
    errormsg.text(xmlParsed['errorcode'] + ': ' + xmlParsed['errorinfo']);
    errormsg.show();
  } else if (xmlParsed['hasException']) {
    errormsg.text(xmlParsed['exceptionMsg']);
    errormsg.show();
  }

  // In case of an error, prevent switching to the details tab
  let certificates_tab = $('a[href="#certificates"]');
  let connection = $('#connection');
  let connection_li = $('#connection-li');
  let connection_tab = $('a[href="#connection"]');
  connection_li.removeClass("disabled");
  connection_tab.attr('data-toggle', 'tab');
  if (!xmlParsed['hasSigInfo']) {
    connection_li.addClass("disabled");
    connection_tab.removeAttr('data-toggle'); // => not clickable
    certificates_tab.tab('show');
  }

  // Tab: Details
  $('#co-detail-certserial').text(cert.serialNumber);
  $('#co-detail-certusage').text(cert.keyUsage);

  let tslheadline = $('#tsl-headline');
  let detailinfo = $('#co-detailinfo');
  detailinfo.hide();
  tslheadline.removeClass('headline');
  tslheadline.addClass('no-connection');
  tslheadline.text('No TSL information available!');
  //tslheadline.hide();

  if (xmlParsed['hasTslInfo']) {
    tslheadline.removeClass('no-connection');
    tslheadline.addClass('headline');
    tslheadline.text('TSL Information');
    //tslheadline.show();
    detailinfo.show();

    $('#co-detail-tsl-issuercountry').text(xmlParsed['tsl_issuercountrycode']);
    $('#co-detail-tsl-servicetypestatus').html('<a href="' + xmlParsed['tsl_servicetypestatus'] + '" target="_blank">' + xmlParsed['tsl_servicetypestatus'] + '</a>');
    $('#co-detail-tsl-servicetypeidentifier').html('<a href="' + xmlParsed['tsl_servicetypeidentifier'] + '" target="_blank">' + xmlParsed['tsl_servicetypeidentifier'] + '</a>');
    $('#co-detail-tsl-additionalinformation').html('<a href="' + xmlParsed['tsl_additionalserviceinformation'] + '" target="_blank">' + xmlParsed['tsl_additionalserviceinformation'] + '</a>');
  }
});

$(document).ready(function() {
  $('a[data-toggle="tab"]').on('shown.bs.tab', function (e) {
    activeTab = $(e.target).attr('aria-controls');
  });
});
