/*
 * Copyright 2017 by A-SIT, Secure Information Technology Center Austria
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */
/**
 * Original code adapted from https://github.com/mozilla/lightbeam/blob/master/lib/tab/utils.js
 */

const { Ci } = require('chrome');
const { getTabForBrowser, getTabForContentWindow, getTabId } = require('sdk/tabs/utils');

// Below code is based on adhacker, taken from http://forums.mozillazine.org/viewtopic.php?f=19&p=6335275
function getTabForChannel(aHttpChannel) {
  let loadContext = getLoadContext(aHttpChannel);
  if (!loadContext) {
    // fallback
    return getTabForChannel2(aHttpChannel);
  }

  return getTabForLoadContext(loadContext);
}

function getUniqueTabId(tab) {
  return getTabId(tab);
}

// Special case in case we don't have a load context.
function getTabForChannel2(aChannel) {
  let win = getWindowForChannel(aChannel);
  if (!win) return null;

  return getTabForContentWindow(win);
}

// Get the tab for a LoadContext
function getTabForLoadContext(aLoadContext) {
  let browser = aLoadContext.topFrameElement;
  if (browser) {
    // Should work in e10s or in non-e10s Firefox >= 39
    return getTabForBrowser(browser);
  }

  // fallback
  return getTabForLoadContext2(aLoadContext);
}

// Fallback for when we can't get the tab for a LoadContext via
// topFrameElement. This happens in:
//    * Firefox < 38, where topFrameElement is not defined when not in e10s.
//    * Firefox 38, where topFrameElement is defined when not in e10s, but
//      the getTabForBrowser call fails
//    * other cases where the tab simply cannot be figured out. This function
//      will return null in these cases.
function getTabForLoadContext2(aLoadContext) {
  try {
    let win = aLoadContext.topWindow;
    if (win) {
      return getTabForContentWindow(win);
    }
  } catch(err1) {
    // requesting aLoadContext.topWindow when in e10s throws an error
  }

  return null;
}

function getLoadContext(aRequest) {
  try {
    // first try the notification callbacks
    return aRequest.QueryInterface(Ci.nsIChannel).notificationCallbacks.getInterface(Ci.nsILoadContext);
  } catch (err1) {
    // fail over to trying the load group
    try {
      if (!aRequest.loadGroup) return null;

      return aRequest.loadGroup.notificationCallbacks.getInterface(Ci.nsILoadContext);
    } catch (err2) {
      return null;
    }
  }
}

function getWindowForChannel(aRequest) {
  let oHttp = aRequest.QueryInterface(Ci.nsIHttpChannel);

  if (!oHttp.notificationCallbacks) {
    //console.log("HTTP request missing callbacks: " + oHttp.originalURI.spec);
    return null;
  }

  let interfaceRequestor = oHttp.notificationCallbacks.QueryInterface(Ci.nsIInterfaceRequestor);
  try {
    return interfaceRequestor.getInterface(Ci.nsIDOMWindow);
  } catch (e) {
    // Happens every time an URL is loaded in background
    //console.log("Failed to to find nsIDOMWindow from interface requestor: " + e);
  }

  return null;
}

exports.getTabForChannel = getTabForChannel;
exports.getUniqueTabId = getUniqueTabId;
exports.getWindowForChannel = getWindowForChannel;