/*
 * Copyright 2017 by A-SIT, Secure Information Technology Center Austria
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */

"use strict";

const { browserWindows } = require('sdk/windows');
const { modelFor } = require('sdk/model/core');
const { viewFor } = require('sdk/view/core');

let defaultButtonId = 0;

function UrlbarButton(options) {
  options = options || {};
  options.id = options.id || 'urlbar-button-' + defaultButtonId++;

  // Show urlbar button in existing windows
  let buttons = [];
  Array.prototype.forEach.call(browserWindows, createButton);
  // Show urlbar button in all new windows
  browserWindows.on('open', createButton);
  browserWindows.on('close', onWindowClose);

  return {
    setImage,
    setVisibility,
    remove
  };

  function remove() {
    buttons.forEach(({win, btn}) => destroyButton(btn));
    buttons = [];
    browserWindows.off('open', createButton);
    browserWindows.off('close', onWindowClose);
  }

  // 1. check if tab is active
  // 2. find window where tab is located
  // 3. find related urlbar button
  // 4. update image of this button
  function setImage(image, tabId) {
    for (let {win, btn} of buttons) {
      if (win.tabs.activeTab.id === tabId) {
        btn.setAttribute('src', image);
      }
    }
  }

  function setVisibility(visible, tabId) {
    for (let {win, btn} of buttons) {
      if (win.tabs.activeTab.id === tabId) {
        btn.setAttribute('collapsed', !visible);
      }
    }
  }

  function createButton(win) {
    /**
     * Alternative: http://stackoverflow.com/questions/25283607
     * btn = doc.createElement('toolbarbutton');
     * btn.setAttribute('image', options.image);        <-- src   | image
     * btn.addEventListener('command', onClick, false); <-- click | command
     */

    let doc = viewFor(win).document
      , urlbarIcons = doc.getElementById('urlbar-icons')
      , btn = doc.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "image");

    btn.setAttribute('id', options.id);
    if (options.image)
      btn.setAttribute('src', options.image);
    if (options.tooltip)
      btn.setAttribute('tooltiptext', options.tooltip);
    if (options.collapsed) {
      btn.setAttribute('collapsed', options.collapsed);
    }

    btn.addEventListener('click', onClick, false);
    urlbarIcons.appendChild(btn);
    buttons.push({win, btn});
  }

  function destroyButton(btn) {
    btn.removeEventListener('click', onClick);
    btn.remove();
  }

  function onWindowClose(win) {
    buttons = buttons.filter(obj => {
      if (obj.win !== win) {
        return true;
      }
      destroyButton(obj.btn);
    });
  }

  function onClick(event) {
    let doc = event.currentTarget;
    for (; doc.parentNode; doc = doc.parentNode);
    let win = modelFor(doc.defaultView);
    if (options.onClick) {
      options.onClick(win.tabs.activeTab.id, event, this);
    }
  }
}

exports.UrlbarButton = UrlbarButton;